
class ResponseHandler():
    """
    Class for handling responses.

    Methods:
        - format_response: Abstract method for formatting responses.
    """

    def format_response(self, result):
        """
        Format responses.

        Parameters:
            - result: Result data to be included in the response.

        Returns:
            Any: Formatted response.
        """
        pass